require_dependency 'application_helper'

module CollapseApplicationHelperPatch
  def self.included(base) # :nodoc:
    base.extend(ClassMethods)

    base.send(:include, InstanceMethods)
  end
  
  module ClassMethods
  end
  
  module InstanceMethods
    
    ###
    # Compatibility helpers
    ###
    
    # Redmine Core 0.8-stable@r2229 (0.8.1)
    # Returns true if the method is defined, else it returns false
    def jump_to_current_view_implemented
      return Redmine::MenuManager::MenuController.method_defined?("redirect_to_project_menu_item")
    end
    
    # Redmine Core 0.8-stable@r2230 (0.8.1)
    # Returns true if the method is defined, else it returns false
    def render_flash_messages_implemented 
      return ApplicationHelper.method_defined?("render_flash_messages")
    end
    
    # Redmine Core trunk@r2304 (0.9-devel)
    # Returns true if the method is defined, else it returns false
    def render_project_hierarchy_implemented 
      return ProjectsHelper.method_defined?("render_project_hierarchy")
    end
    
    # Redmine Core trunk@r2485 (0.9-devel)
    # Returns true if the method is defined, else it returns false
    def page_header_title_implemented
      return ApplicationHelper.method_defined?("page_header_title")
    end
    
    # Redmine Core trunk@r2493 (0.9-devel)
    # Returns true if the constant is defined, else it returns false
    def rails_i18n_implemented
      return Redmine.const_defined?(:I18n)
    end
    
    ###
    # Tab helpers
    ###
	
    def show_left_menu
	return User.current.logged?
    end
    
    # Left-menu tabs renderer
    # Returns an array named 'tabs' containing the left-menu tabs to render
    def left_menu_tabs
      # Don't render the menus-tab nor the projects-tab unless specifically configured
      if Setting.plugin_redmine_collapse['show_menus_tab'] == '1' && Setting.plugin_redmine_collapse['show_projects_tab'] == '1'
	if has_content?(:sidebar)
          tabs = [ { :name => 'projects-tab', :label => localize(:label_projects_tab), :partial => 'left_menu/projects.rhtml' },
                   { :name => 'actions-tab', :label => localize(:label_actions_tab), :partial => 'left_menu/actions.rhtml' },		
                   { :name => 'menus-tab', :label => localize(:label_menus_tab), :partial => 'left_menu/menus.rhtml' }]
	else
	  tabs = [ { :name => 'projects-tab', :label => localize(:label_projects_tab), :partial => 'left_menu/projects.rhtml' },
                   { :name => 'menus-tab', :label => localize(:label_menus_tab), :partial => 'left_menu/menus.rhtml' }]
	end	
      # Don't render the menus-tab unless specifically configured
      elsif Setting.plugin_redmine_collapse['show_menus_tab'] == '1'
	if has_content?(:sidebar)
          tabs = [ { :name => 'actions-tab', :label => localize(:label_actions_tab), :partial => 'left_menu/actions.rhtml' },
                   { :name => 'menus-tab', :label => localize(:label_menus_tab), :partial => 'left_menu/menus.rhtml' } ]
	else
          tabs = [ { :name => 'menus-tab', :label => localize(:label_menus_tab), :partial => 'left_menu/menus.rhtml' } ]
	end
      # Don't render the projects-tab unless specifically configured
      elsif Setting.plugin_redmine_collapse['show_projects_tab'] == '1'
	if has_content?(:sidebar)
       	  tabs = [ { :name => 'projects-tab', :label => localize(:label_projects_tab), :partial => 'left_menu/projects.rhtml' } ,
		   { :name => 'actions-tab', :label => localize(:label_actions_tab), :partial => 'left_menu/actions.rhtml' } ]
	else
       	  tabs = [ { :name => 'projects-tab', :label => localize(:label_projects_tab), :partial => 'left_menu/projects.rhtml' }]
	end
      # Always render the actions-tab
      else
        if has_content?(:sidebar)
          tabs = [ { :name => 'actions-tab', :label => localize(:label_actions_tab), :partial => 'left_menu/actions.rhtml' } ]
        end
      end
	
      
      return tabs
    end

    def localize(name)	
	if rails_i18n_implemented 
          return name
	else
	  return l(name)
	end
    end
    
    # Left-menu selected-tab based on active controller and activated-tabs settings
    def left_menu_selected_tab      
      return 'projects-tab'
    end
    
    ###
    # Tab design helpers
    ###

    # List of additional themes that are supported by the Collapse plugin besides the default Redmine theme.
    # Others will use the 'default' Collapse plugin style.
    def supported_themes
      [
       'alternate',
       'classic',
       'basecamp',
       'squeejee'
      ]
    end
    
    # Determine the currently-used Redmine theme based on the global setting
    # Returns a string named 'currenttheme' containing the name of the current theme
    def current_redmine_theme
      if supported_themes.include?(Setting.ui_theme)
        currenttheme = Setting.ui_theme
      else
        currenttheme = 'default'
      end
      
      return currenttheme
    end
    
    # Renders the CSS-style used for selected project menu items in the projects-tab
    # Returns a string containing the style-name or nil
    def project_menu_item_class(selected=false)
      if selected
        'selected'
      else
        ''
      end
    end
    
    # Renders the CSS-style used for selected global menu items in the menus-tab
    # Returns a string containing the style-name or nil
    def global_menu_item_class(selected=false)
      if selected
        'alt-selected'
      else
        ''
      end
    end
    
    ###
    # Tab content helpers
    ###
    
    # Renders the projects tree with collapse
    def render_projects_nestedset(projects)
      s = "<table>"
      s << "<tbody>"
      ancestors = []
      for project in projects
      	rowid = ""
      	classes = ""
      	spanicon = ""
      	openonclick = ""
      	showchildren = false
        project.children.each do |child|
		if projects.include?(child)
		    showchildren = true
		    break
		end
	end
	if(!project.children.empty? && showchildren)
		classes += " closed parent " #+ cycle("odd", "even")
		rowid = "id=\""+project.id.to_s+"span\""
		openonclick = "onclick=\"showHide('"+project.id.to_s+"','"+project.id.to_s+"span')\""
		spanicon = "<span " + openonclick + " class=\"expander\">&nbsp</span>"
	else
		classes += " child"
	end
	if(project.parent_id == nil)
		ancestors.clear
		ancestors << project.id
	else
		while (ancestors.any? && !(project.parent_id == ancestors.last))
			ancestors.pop
		end 
		classes += " hide"
		if( !(ancestors.detect {|pid| pid == project.parent_id }))
			prvclasses = "closed show parent " + cycle("odd", "even")
			ancestors.each do |pid|
				prvclasses += " " + pid.to_s
			end
			openonclick = "onclick=\"showHide('"+project.parent_id.to_s+"','"+project.parent_id.to_s+"span')\""
			s<< "<tr class=\"" + prvclasses  + "\" id=" + project.parent_id.to_s + "span>" +
				"<td class=\"name\" >" + 
				"<span style=\"padding-left: " + (2*(ancestors.length-1)).to_s + "em;\"></span>" 
				"<span " + openonclick + " class=\"expander\" >&nbsp</span></tr>"
			ancestors << project.parent_id
		end
		ancestors.each do |pid|
		        classes += " " + pid.to_s
		end
		ancestors << project.id
	end	   
	s << "<tr class=\"" +  classes + "\""  + rowid + ">" + 
		"<td class=\"name\">" + "<span style=\"padding-left: " + (2*(ancestors.length-1)).to_s + "em;\"></span>" +
		spanicon  + (project.active? ? link_to(h(project.name), {:controller => 'projects', :action => 'show', :id => project}, :class => "project") : h(project.name)) + 
		"<span " + openonclick + " class=\"empty " + (User.current.member_of?(project) ? "my-project" : "") +"\">&nbsp</span></td>"
      end
      s << "</tr></tbody></table>"
      return s
    end

    # Renders the projects tree without collapse
    def render_projects(projects)
      s = ''
      if projects.any?
        ancestors = []
        # Iterate over the projects
        projects.each do |project|
          if (ancestors.empty? || project.is_descendant_of?(ancestors.last))
            s << "<ul class='projects #{ ancestors.empty? ? 'root' : nil}'>\n"
          else
            ancestors.pop
            s << "</li>"
            while (ancestors.any? && !project.is_descendant_of?(ancestors.last)) 
              ancestors.pop
              s << "</ul></li>\n"
            end
          end
          classes = (ancestors.empty? ? 'root' : 'child')
          s << "<li>" + "<div class=\"empty " + (User.current.member_of?(project) ? "my-project" : "") + "\">" +
                      # Only add ':jump => current_menu_item' URL-parameter if on project level
                      if !@project.nil?
                        # Add 'selected'-class if the following condition is true
                        project_selected = project.identifier == @project.identifier
                        link_to(h(project), {:controller => 'projects', :action => 'show', :id => project, :jump => current_menu_item}, :class => project_menu_item_class(project_selected))
                      else
                        link_to(h(project), {:controller => 'projects', :action => 'show', :id => project})
                      end
          s << "</div>"
          ancestors << project
        end        
        s << ("</li></ul>\n" * ancestors.size)
      end
      return s
    end
        
    # Renders the global menu as an unordered list (relies on backwards-compatibility in the core for the RESTful-routes of Redmine 0.9.x)
    # Returns an array named 'globalmenu' containing the HTML for the global menu section in the menus-tab
    def render_global_menu
      returning '' do |globalmenu|
        globalmenu << "<ul>\n"
        
        ## Global activity link ##
        # 'alt-selected'-class is added if the defined condition is true
        activity_selected = params[:controller] == 'projects' && params[:action] == 'activity' && params[:id].nil?
        globalmenu << content_tag(:li, link_to(l(:label_activity), { :controller => 'projects', :action => 'activity', :id => nil }, :class => global_menu_item_class(activity_selected)))
        
        ## Global issues link ##
        # 'alt-selected'-class is added if the defined condition is true
        issues_selected = params[:controller] == 'issues' && params[:action] == 'index' && params[:project_id].nil?
        globalmenu << content_tag(:li, link_to(l(:label_issue_plural), { :controller => 'issues', :id => nil }, :class => global_menu_item_class(issues_selected)))
        
        ## Global calendar link ##
        if User.current.allowed_to?(:view_calendar, @project, :global => true)
          # 'alt-selected'-class is added if the defined condition is true
          calendar_selected = params[:controller] == 'issues' && params[:action] == 'calendar' && params[:project_id].nil?
          globalmenu << content_tag(:li, link_to(l(:label_calendar), { :controller => 'issues', :action => 'calendar', :project_id => nil }, :class => global_menu_item_class(calendar_selected)))
        end
        
        ## Global gantt link ##
        if User.current.allowed_to?(:view_gantt, @project, :global => true)
          # 'alt-selected'-class is added if the defined condition is true
          gantt_selected = params[:controller] == 'issues' && params[:action] == 'gantt' && params[:project_id].nil?
          globalmenu << content_tag(:li, link_to(l(:label_gantt), { :controller => 'issues', :action => 'gantt', :project_id => nil }, :class => global_menu_item_class(gantt_selected)))
        end
        
        ## Global spent-time details link ##
        if User.current.allowed_to?(:view_time_entries, @project, :global => true)
          # 'alt-selected'-class is added if the defined condition is true
          spent_time_details_selected = params[:controller] == 'timelog' && params[:action] == 'details' && params[:project_id].nil?
          globalmenu << content_tag(:li, link_to(l(:label_spenttime_details), { :controller => 'timelog', :action => 'details', :project_id => nil }, :class => global_menu_item_class(spent_time_details_selected)))
        end
        
        ## Global spent-time report link ##
        if User.current.allowed_to?(:view_time_entries, @project, :global => true)
          # 'alt-selected'-class is added if the defined condition is true
          spent_time_report_selected = params[:controller] == 'timelog' && params[:action] == 'report' && params[:project_id].nil?
          globalmenu << content_tag(:li, link_to(l(:label_spenttime_report), { :controller => 'timelog', :action => 'report', :project_id => nil }, :class => global_menu_item_class(spent_time_report_selected)))
        end
        
        ## Global news link ##
        if User.current.allowed_to?(:view_news, @project, :global => true)
          # 'alt-selected'-class is added if the defined condition is true
          news_selected = params[:controller] == 'news' && params[:action] == 'index' && params[:project_id].nil?
          globalmenu << content_tag(:li, link_to(l(:label_news), { :controller => 'news', :action => 'index', :project_id => nil }, :class => global_menu_item_class(news_selected)))
        end
        
        globalmenu << "</ul>\n"
      end
    end    
  end # Close the module CollapseApplicationHelperPatch::InstanceMethods
end # Close the module CollapseApplicationHelperPatch
